@@ -65,28 +65,43 @@ static gboolean cb_expose(GtkWidget *wid, GdkEventExpose *e, gContainer *data)
 
 static void cb_map(GtkWidget *widget, gContainer *sender)
 {
+	//fprintf(stderr, "cb_map: %s %d\n", sender->name(), gApplication::_disable_mapping_events);
+
 	sender->setShown(true);
-	if (!sender->isTempHidden())
-		sender->arrangeLater();
+	sender->arrangeLater();
 }
 
+#if GTK3
+static void cb_remap_children(GtkWidget *widget, GdkEvent *event, gContainer *sender)
+{
+	int i;
+	gControl *child;
+	
+	//fprintf(stderr, "cb_remap_children: '%s' <<<\n", sender->name());
+	
+	for (i = 0; i < sender->childCount(); i++)
+	{
+		child = sender->child(i);
+		if (gtk_widget_get_visible(child->border))
+		{
+			child->hideButKeepFocus();
+			child->showButKeepFocus();
+		}
+	}
+
+	//fprintf(stderr, "cb_remap_children: >>> '%s'\n", sender->name());
+}
+#endif
+
 static void cb_unmap(GtkWidget *widget, gContainer *sender)
 {
+	//fprintf(stderr, "cb_unmap: %s %d\n", sender->name(), gApplication::_disable_mapping_events);
+	if (gApplication::_disable_mapping_events)
+		return;
+
 	sender->setShown(false);
 }
 
-static void cb_arrange(gContainer *sender)
-{
-	if (sender->onArrange)
-		(*(sender->onArrange))(sender);
-}
-
-static void cb_before_arrange(gContainer *sender)
-{
-	if (sender->onBeforeArrange)
-		(*(sender->onBeforeArrange))(sender);
-}
-
 static void resize_container(gContainer *cont, int w, int h)
 {
 	/*w += cont->width() - cont->containerWidth();
@@ -100,13 +115,14 @@ static void resize_container(gContainer *cont, int w, int h)
 #define WIDGET_TYPE gControl*
 #define CONTAINER_TYPE gContainer*
 #define ARRANGEMENT_TYPE gContainerArrangement*
-#define IS_RIGHT_TO_LEFT() gDesktop::rightToLeft()
+#define IS_RIGHT_TO_LEFT(_object) (gtk_widget_get_direction(((gControl *)_object)->widget) == GTK_TEXT_DIR_RTL)
 #define GET_WIDGET(_object) _object
 #define GET_CONTAINER(_object) _object
 #define GET_ARRANGEMENT(_object) (((gContainer*)_object)->getArrangement())
 #define IS_EXPAND(_object) (((gControl*)_object)->isExpand())
 #define IS_IGNORE(_object) (((gControl*)_object)->isIgnore())
 #define IS_DESIGN(_object) (((gControl*)_object)->isDesign())
+#define IS_USER(_object) (((gContainer *)_object)->isUser())
 #define IS_WIDGET_VISIBLE(_widget)  (((gControl*)_widget)->isVisible())
 
 #define CAN_ARRANGE(_object) (((gContainer *)_object)->isShown() && !((gControl *)_object)->isDestroyed())
@@ -143,8 +159,8 @@ static void resize_container(gContainer *cont, int w, int h)
 
 #define GET_OBJECT_NAME(_object) (((gControl *)_object)->name())
 
-#define RAISE_ARRANGE_EVENT(_object) cb_arrange((gContainer *)_object);
-#define RAISE_BEFORE_ARRANGE_EVENT(_object) cb_before_arrange((gContainer *)_object);
+#define RAISE_ARRANGE_EVENT(_object) CB_container_arrange((gContainer *)_object);
+#define RAISE_BEFORE_ARRANGE_EVENT(_object) CB_container_before_arrange((gContainer *)_object);
 
 #define DESKTOP_SCALE gDesktop::scale()
 
@@ -159,6 +175,7 @@ void gContainer::performArrange()
 	else
 	{
 		_did_arrangement = false;
+		//fprintf(stderr, "performArrange: %p / %p [%d] '%s'\n", this, hFree, isShown(), name());
 		arrangeContainer((void *)this);
 	}
 }
@@ -267,8 +284,6 @@ void gContainer::initialize()
 	_children = g_ptr_array_new();
 	
 	radiogroup = NULL;
-	onArrange = NULL;
-	onBeforeArrange = NULL;
 	
 	_proxyContainer = NULL;
 	_proxyContainerFor = NULL;
@@ -279,6 +294,7 @@ void gContainer::initialize()
 	_no_arrangement = 0;
 	_did_arrangement = false;
 	_is_container = true;
+	_user = false;
 	_user_container = false;
 	_shown = false;
 	_arrange_later = false;
@@ -288,12 +304,10 @@ void gContainer::initialize()
 	arrangement.padding = 0;
 	arrangement.autoresize = false;
 	arrangement.locked = false;
-	arrangement.user = false;
 	arrangement.margin = false;
 	arrangement.indent = false;
 	arrangement.centered = false;
 	arrangement.invert = false;
-	arrangement.paint = false;
 }
 
 gContainer::gContainer() 
@@ -426,18 +440,10 @@ void gContainer::setAutoResize(bool vl)
 
 void gContainer::setUser()
 {
-	if (arrangement.user)
-		return;
-	
-	arrangement.user = true;
+	_user = true;
 	performArrange();
 	updateDesignChildren();
-}
-
-void gContainer::setPaint()
-{
-	arrangement.paint = true;
-	ON_DRAW(border, this, cb_expose, cb_draw);
+	ON_DRAW_BEFORE(border, this, cb_expose, cb_draw);
 }
 
 void gContainer::setInvert(bool vl)
@@ -653,6 +659,8 @@ void gContainer::insert(gControl *child, bool realize)
 	
 	if ((isUser() && isDesign()) || isDesignIgnore())
 		child->setDesign(true);
+	
+	child->updateDirection();
 }
 
 void gContainer::remove(gControl *child)
@@ -746,10 +754,15 @@ GtkWidget *gContainer::getContainer()
 	return widget;
 }
 
-void gContainer::connectBorder()
+void gContainer::borderSignals()
 {
-	g_signal_connect_after(G_OBJECT(border), "map", G_CALLBACK(cb_map), (gpointer)this);	
-	g_signal_connect_after(G_OBJECT(border), "unmap", G_CALLBACK(cb_unmap), (gpointer)this);	
+	gControl::borderSignals();
+	
+	g_signal_connect(G_OBJECT(border), "map", G_CALLBACK(cb_map), (gpointer)this);	
+	g_signal_connect(G_OBJECT(border), "unmap", G_CALLBACK(cb_unmap), (gpointer)this);	
+#ifdef GTK3
+	g_signal_connect_after(G_OBJECT(border), "map-event", G_CALLBACK(cb_remap_children), (gpointer)this);	
+#endif
 }
 
 bool gContainer::resize(int w, int h, bool no_decide)
@@ -760,6 +773,9 @@ bool gContainer::resize(int w, int h, bool no_decide)
 	_client_w = 0;
 	_client_h = 0;
 	
+	if (isUser())
+		CUSERCONTROL_cb_resize(this);
+
 	performArrange();
 	return false;
 }
@@ -807,7 +823,7 @@ void gContainer::updateFont()
 	for (i = 0; i < childCount(); i++)
 		child(i)->updateFont();
 
-	if (arrangement.paint)
+	if (isUser())
 		CUSERCONTROL_cb_font(this);
 }
 
@@ -936,6 +952,8 @@ void gContainer::resetArrangeLater()
 {
 	if (_arrange_later)
 	{
+		//fprintf(stderr, "resetArrangeLater: %p / %p '%s'\n", this, hFree, name());
+		
 		_arrange_later = false;
 		_arrange_list = g_list_remove(_arrange_list, this);
 	}
@@ -945,6 +963,8 @@ void gContainer::arrangeLater()
 {
 	if (!_arrange_later)
 	{
+		//fprintf(stderr, "arrangeLater: %p / %p '%s'\n", this, hFree, name());
+	
 		_arrange_later = true;
 		_arrange_list = g_list_prepend(_arrange_list, (gpointer)this);
 	}
@@ -967,5 +987,24 @@ void gContainer::postArrange()
 		cont->performArrange();
 	}
 
+	g_list_free(_arrange_list);
 	_arrange_list = NULL;
 }
+
+void gContainer::updateDirection()
+{
+	int i;
+	gControl *child;
+	
+	gControl::updateDirection();
+	
+	for (i = 0;; i++)
+	{
+		child = gContainer::child(i);
+		if (!child)
+			break;
+		child->updateDirection();
+	}
+	
+	performArrange();
+}
