@@ -25,6 +25,8 @@
 
 #define __C_MEDIA_C
 
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include "c_media.h"
 //#include <gst/interfaces/xoverlay.h>
 #include <gst/base/gstbasesink.h>
@@ -142,8 +144,6 @@ void MEDIA_set_flag(void *element, char *property, int flag, bool value)
 	g_object_set(G_OBJECT(element), property, flags, NULL);
 }
 
-
-
 static GB_TYPE to_gambas_type(const GValue *value)
 {
 	switch (G_VALUE_TYPE(value))
@@ -285,6 +285,7 @@ static void return_value(const GValue *value)
 	  GB.ReturnNull();
 	  return;
 	}
+
 	GType type = G_VALUE_TYPE(value);
 	
 	switch (type)
@@ -374,7 +375,7 @@ static void return_value(const GValue *value)
 
 				GB.ReturnObject(array);
 			}
-#if defined(G_TYPE_VALUE_ARRAY) && !GLIB_CHECK_VERSION(2, 32, 0)
+#if defined(G_TYPE_VALUE_ARRAY) //&& !GLIB_CHECK_VERSION(2, 32, 0)
 			else if (G_VALUE_HOLDS(value, G_TYPE_VALUE_ARRAY))
 			{
 				GValueArray *garray = (GValueArray *)g_value_get_boxed(value);
@@ -765,6 +766,13 @@ static void set_pipeline_rate(void *_object)
 	THIS_PIPELINE->rate = THIS_PIPELINE->next_rate;	
 }
 
+static gint64 get_pipeline_duration(void *_object)
+{
+	if (THIS_PIPELINE->duration < 0)
+		gst_element_query_duration(ELEMENT, GST_FORMAT_TIME, &THIS_PIPELINE->duration);
+	return THIS_PIPELINE->duration;
+}
+
 #if 0
 //---- MediaSignalArguments -----------------------------------------------
 
@@ -871,7 +879,7 @@ END_PROPERTY
 
 //---- MediaMessage -------------------------------------------------------
 
-#define MESSAGE_DATA (gst_message_get_structure(THIS_MESSAGE->message))
+#define MESSAGE_DATA (THIS_MESSAGE->structure)
 
 static int MediaMessage_check(void *_object)
 {
@@ -884,15 +892,23 @@ static CMEDIAMESSAGE *create_message(GstMessage *message)
 	
 	ob = GB.New(GB.FindClass("MediaMessage"), NULL, NULL);
 	ob->message = message;
+	ob->structure = gst_message_get_structure(message);
 	ob->lastKey = NULL;
 	return ob;
 }
 
 BEGIN_METHOD(MediaMessage_get, GB_STRING name)
 
-	char *name = GB.ToZeroString(ARG(name));
+	char *name;
 	const GValue *value;
 
+	if (!MESSAGE_DATA)
+	{
+		GB.ReturnNull();
+		return;
+	}
+
+	name = GB.ToZeroString(ARG(name));
 	value = gst_structure_get_value(MESSAGE_DATA, name);
 	return_value(value);
 	
@@ -906,7 +922,7 @@ BEGIN_PROPERTY(MediaMessage_Keys)
 	const GstStructure *data = MESSAGE_DATA;
 	int nfields, i;
 	
-	nfields = gst_structure_n_fields(data);
+	nfields = data ? gst_structure_n_fields(data) : 0;
 	
 	GB.Array.New(&array, GB_T_STRING, nfields);
 	
@@ -918,47 +934,58 @@ BEGIN_PROPERTY(MediaMessage_Keys)
 END_PROPERTY
 
 BEGIN_PROPERTY(MediaMessage_Type)
-    
-    GB.ReturnInteger(GST_MESSAGE_TYPE(THIS_MESSAGE->message));
-    
+
+  GB.ReturnInteger(GST_MESSAGE_TYPE(THIS_MESSAGE->message));
+
 END_PROPERTY
 
+/*
+BEGIN_PROPERTY(MediaMessage_TypeName)
+
+  GB.ReturnConstZeroString(gst_message_type_get_name(GST_MESSAGE_TYPE(THIS_MESSAGE->message)));
+
+END_PROPERTY
+*/
+
 BEGIN_PROPERTY(MediaMessage_Name)
-    
-    GB.ReturnNewZeroString(gst_structure_get_name(MESSAGE_DATA));
+
+	if (MESSAGE_DATA)
+		GB.ReturnNewZeroString(gst_structure_get_name(MESSAGE_DATA));
+	else
+		GB.ReturnVoidString();
     
 END_PROPERTY
 
 BEGIN_PROPERTY(MediaMessage_Count)
-    
-    GB.ReturnInteger(gst_structure_n_fields(MESSAGE_DATA));
-    
+
+	GB.ReturnInteger(MESSAGE_DATA ? gst_structure_n_fields(MESSAGE_DATA) : 0);
+
 END_PROPERTY
 
 BEGIN_PROPERTY(MediaMessage_Key)
     
-    GB.ReturnNewZeroString(THIS_MESSAGE->lastKey);
+	GB.ReturnNewZeroString(THIS_MESSAGE->lastKey);
     
 END_PROPERTY
 
 BEGIN_METHOD_VOID(MediaMessage_next)
     
-    const GstStructure *data = MESSAGE_DATA;
-    int count = gst_structure_n_fields(data);
-    int *index = (int *)GB.GetEnum();
+	const GstStructure *data = MESSAGE_DATA;
+	int count = data ? gst_structure_n_fields(data) : 0;
+	int *index = (int *)GB.GetEnum();
+
+	if (*index < 0 || *index >= count)
+		GB.StopEnum();
+	else
+	{
+		THIS_MESSAGE->lastKey = gst_structure_nth_field_name(data, *index);
+		const GValue *value = gst_structure_get_value(data, THIS_MESSAGE->lastKey);
+		return_value(value);
+
+		GB.ReturnConvVariant();
+		(*index)++;
+	}
 
-    if (*index < 0 || *index >= count)
-            GB.StopEnum();
-    else
-    {
-            THIS_MESSAGE->lastKey = gst_structure_nth_field_name(data, *index);
-            const GValue *value = gst_structure_get_value(data, THIS_MESSAGE->lastKey);
-            return_value(value);
-            
-            GB.ReturnConvVariant();
-            (*index)++;
-    }
-    
 END_PROPERTY
 
 //---- MediaLink ----------------------------------------------------------
@@ -1058,12 +1085,28 @@ static MEDIA_TYPE _types[] =
 static void cb_pad_added(GstElement *element, GstPad *pad, CMEDIACONTROL *_object)
 {
 	char *name;
+	CMEDIACONTROL *target;
 
-	if (!THIS->dest)
+	if (!THIS->dest || GB.Count(THIS->dest) == 0)
 		return;
 	
+	target = (CMEDIACONTROL *)THIS->dest[0];
+
 	name = gst_pad_get_name(pad);
-	gst_element_link_pads(ELEMENT, name, ((CMEDIACONTROL *)THIS->dest)->elt, NULL);
+
+	if (!gst_element_link_pads(ELEMENT, name, target->elt, NULL))
+	{
+		if (MAIN_debug)
+			fprintf(stderr, "gb.media: warning: unable to link later element '%s' to output '%s.%s'\n", gst_element_get_name(target->elt), gst_element_get_name(ELEMENT), name);
+	}
+	else
+	{
+		if (MAIN_debug)
+			fprintf(stderr, "gb.media: info: link later element '%s' to output '%s.%s'\n", gst_element_get_name(target->elt), gst_element_get_name(ELEMENT), name);
+		GB.Unref(POINTER(&target));
+		GB.Remove(&THIS->dest, 0, 1);
+	}
+
 	g_free(name);
 }
 
@@ -1161,8 +1204,7 @@ BEGIN_METHOD_VOID(MediaControl_free)
 
 	//fprintf(stderr, "MediaControl_free: %p\n", THIS);
 
-	GB.Unref(POINTER(&THIS->dest));
-	//GB.FreeString(&THIS->type);
+	GB.FreeArray(POINTER(&THIS->dest));
 	GB.StoreVariant(NULL, &THIS->tag);
 	
 	if (ELEMENT)
@@ -1304,10 +1346,14 @@ BEGIN_METHOD(MediaControl_LinkLaterTo, GB_OBJECT dest)
 	if (GB.CheckObject(dest))
 		return;
 
-	GB.Unref(POINTER(&THIS->dest));
+	if (!THIS->dest)
+	{
+		GB.NewArray(&THIS->dest, sizeof(void *), 0);
+		g_signal_connect(ELEMENT, "pad-added", G_CALLBACK(cb_pad_added), THIS);
+	}
+
 	GB.Ref(dest);
-	THIS->dest = dest;
-	g_signal_connect(ELEMENT, "pad-added", G_CALLBACK(cb_pad_added), THIS);
+	*(void **)GB.Add(&THIS->dest) = dest;
 
 END_METHOD
 
@@ -1722,6 +1768,7 @@ static int cb_message(CMEDIAPIPELINE *_object)
 				GB.Raise(THIS, EVENT_Event, 1, GB_T_OBJECT, ob);
 				
 				ob->message = NULL;
+				ob->structure = NULL;
 				GB.Unref(POINTER(&ob));
 			}
 			
@@ -1794,13 +1841,17 @@ static int cb_message(CMEDIAPIPELINE *_object)
 					break;
 				}
 				case GST_MESSAGE_BUFFERING: GB.Raise(THIS, EVENT_Buffering, 0); break;
-				case GST_MESSAGE_DURATION_CHANGED: GB.Raise(THIS, EVENT_Duration, 0); break;
+
+				case GST_MESSAGE_DURATION_CHANGED:
+					THIS_PIPELINE->duration = -1;
+					GB.Raise(THIS, EVENT_Duration, 0);
+					break;
+
 				case GST_MESSAGE_PROGRESS: GB.Raise(THIS, EVENT_Progress, 0); break;
 				
 				case GST_MESSAGE_STREAM_START:
 					THIS_PIPELINE->about_to_finish = FALSE;
 					THIS_PIPELINE->pos = 0;
-					gst_element_query_duration(ELEMENT, GST_FORMAT_TIME, &THIS_PIPELINE->duration);
 					GB.Raise(THIS, EVENT_Start, 0);
 					break;
 						
@@ -1824,7 +1875,7 @@ static int cb_message(CMEDIAPIPELINE *_object)
 			THIS_PIPELINE->pos = pos;
 			GB.Raise(THIS, EVENT_Position, 0);
 			//fprintf(stderr, "%" PRId64 " / %" PRId64 "\n", THIS_PIPELINE->pos, THIS_PIPELINE->duration);
-			if (!THIS_PIPELINE->about_to_finish && THIS_PIPELINE->pos > (THIS_PIPELINE->duration - 2000000000))
+			if (!THIS_PIPELINE->about_to_finish && THIS_PIPELINE->pos > (get_pipeline_duration(THIS_PIPELINE) - 2000000000))
 			{
 				THIS_PIPELINE->about_to_finish = TRUE;
 				GB.Raise(THIS, EVENT_AboutToFinish, 0);
@@ -1880,6 +1931,7 @@ BEGIN_METHOD(MediaPipeline_new, GB_INTEGER polling)
 	}
 	
 	THIS_PIPELINE->rate = THIS_PIPELINE->next_rate = 1.0;
+	THIS_PIPELINE->duration = -1;
 
 END_METHOD
 
@@ -1955,7 +2007,7 @@ END_PROPERTY
 
 BEGIN_PROPERTY(MediaPipeline_Duration)
 
-	GB.ReturnFloat((double)(THIS_PIPELINE->duration / 1000) / 1E6);
+	GB.ReturnFloat((double)(get_pipeline_duration(THIS_PIPELINE) / 1000) / 1E6);
 
 END_PROPERTY
 
@@ -2057,6 +2109,7 @@ GB_DESC MediaMessageDesc[] =
 	GB_HOOK_CHECK(MediaMessage_check),
 	
 	GB_PROPERTY_READ("Type", "i", MediaMessage_Type),
+	//GB_PROPERTY_READ("TypeName", "s", MediaMessage_TypeName),
 	GB_PROPERTY_READ("Name", "s", MediaMessage_Name),
 	
 	GB_METHOD("_get", "v", MediaMessage_get, "(Name)s"),
